@@ -1,5 +1,5 @@
 from .gateway import *
 from .http import *
 from .models import *
-from .listener import *
+from .dispatch import *
 from .error import *
