@@ -4,3 +4,8 @@ class HTTPException(Exception):
         self.message = message
 
         super().__init__(f"HTTP error with code: {self.code}!\nMessage: {self.message}")
+
+
+class IsEmpty(Exception):
+    def __init__(self) -> None:
+        super().__init__(f"Dict cannot be empty!")
