@@ -1,47 +1,50 @@
 from asyncio import get_event_loop, get_running_loop, new_event_loop
-from json import loads
-from typing import List
+from typing import List, Union, Optional
 from logging import getLogger
 from sys import version_info
 
 from aiohttp import WSMessage, ClientWebSocketResponse, WSMsgType
 from aiohttp.http import WS_CLOSED_MESSAGE
 
-from .events import TitleUpdateEvent, PlayListUpdateEvent, EncodeEvent, EventType, TorrentUpdateEvent
-from ..http import HTTPCLient
-from ..listener import EventListener
+from .events import (
+    TitleUpdateEvent,
+    PlayListUpdateEvent,
+    EncodeEvent,
+    EventType,
+    TorrentUpdateEvent,
+    TitleSerieEvent,
+)
+from ..http import HTTPClient
+from ..dispatch import EventDispatcher
 
 
 log = getLogger("anilibria.gateway")
 URL = "ws://api.anilibria.tv/v2/ws/"
+__all__ = ["WebSocketClient"]
 
 
 class WebSocketClient:
-    def __init__(self, proxy: str = None):
-        """
-        Клиент для управления вебсокетом.
+    """
+    Клиент для управления вебсокетом.
+    """
 
-        :param proxy: Ссылка на прокси-сервер.
-        """
+    def __init__(self, proxy: str = None):
         try:
             self._loop = get_event_loop() if version_info < (3, 10) else get_running_loop()
         except RuntimeError:
             self._loop = new_event_loop()
-        self._http = HTTPCLient(proxy)
-        self._listener = EventListener()
+        self._http = HTTPClient(proxy)
+        self._listener = EventDispatcher()
         self.proxy: str = proxy
         self._client: ClientWebSocketResponse = None
         self._closed: bool = False
         self._subscribes: List[dict] = None
         self.api_version: str = None
 
-    async def run(self, subscribes: List[dict]):
+    async def run(self):
         """
         Запускает вебсокет.
-
-        :param subscribes: Список с данными о тайтлах, на которые вебсокет подпишется при запуске.
         """
-        self._subscribes = subscribes
         await self.__connect()
 
     async def __connect(self):
@@ -54,60 +57,37 @@ class WebSocketClient:
         async with self._http.request.session.ws_connect(URL) as self._client:
             log.debug("Connected to websocket")
             self._closed = self._client._closed
-            if self._subscribes:
-                await self.__subscribe_on_titles()
-
             if self._closed:
                 await self.__connect()
 
             while not self._closed:
-                packet = await self._client.receive()
-                if self._client is None or packet in [WSMsgType.CLOSE, WS_CLOSED_MESSAGE]:
+                data = await self.__receive_packet_data()
+                if data is None:
+                    continue
+                if self._client is None or data == WS_CLOSED_MESSAGE:
                     await self.__connect()
                     break
-                await self._process_packet(packet)
 
-    async def _process_packet(self, packet: WSMessage):
+                await self.__dispatch_events(data)
+
+    async def __receive_packet_data(self) -> Optional[Union[dict, WSMessage]]:
         """
-        Принимает пакет и диспатчит ивенты.
+        Принимает пакет данных и возвращает словарь с данными или пустое значение
 
         .. warning::
            Не пытайтесь самостоятельно использовать этот метод!
 
-        :param packet: Пакет
-        :type packet: WSMessage
+        :return:
         """
-        data = loads(packet.data)
-        self._listener.dispatch("on_raw_packet", data)
+        packet: WSMessage = await self._client.receive()
+        if packet.type == WSMsgType.CLOSED:
+            return WS_CLOSED_MESSAGE
 
-        type = data.get("type")
-        if type is None:
-            return await self._process_other_events(data)
+        return packet.json() if packet.data and isinstance(packet.data, str) else None
 
-        event_name = f"on_{type}"
-        if type == EventType.TITLE_UPDATE:
-            self._dispatch_title_update_event(data["title_update"])
-        elif type == EventType.PLAYLIST_UPDATE:
-            event = PlayListUpdateEvent(**data[type])
-            self._listener.dispatch(event_name, event)
-        elif type in [
-            EventType.ENCODE_START,
-            EventType.ENCODE_END,
-            EventType.ENCODE_PROGRESS,
-            EventType.ENCODE_FINISH
-        ]:
-            event = EncodeEvent(**data[type])
-            self._listener.dispatch(event_name, event)
-        elif type == EventType.TORRENT_UPDATE:
-            event = TorrentUpdateEvent(**data[type])
-            self._listener.dispatch(event_name, event)
-        else:
-            self._listener.dispatch(event_name, data)
-            log.debug(f"Not documented event type {type} dispatched with data: {data}")
-
-    def _dispatch_title_update_event(self, data: dict):
+    async def __dispatch_events(self, data: dict):
         """
-        Диспачит ивенты ``on_title_update`` и ``on_title``.
+        Принимает словарь с данными об ивенте и диспатчит их.
 
         .. warning::
            Не пытайтесь самостоятельно использовать этот метод!
@@ -115,20 +95,54 @@ class WebSocketClient:
         :param data: Словарь с данными об ивенте
         :type data: dict
         """
-        event_model = TitleUpdateEvent(**data)
-        self._listener.dispatch("on_title_update", event_model)
+        self._listener.dispatch("on_raw_packet", data)
 
-        title_data = data["title"]
-        events = self._listener.events
-        for event in events:
-            if event == "on_title":
-                coro_data = events[event]["data"]
-                if all(value == title_data[key] for key, value in coro_data.items()):
-                    self._listener.dispatch("on_title", event_model)
+        type = data.get("type")
+        if type is None:
+            return await self.__dispatch_other_events(data)
 
-    async def _process_other_events(self, data: dict):
+        event_name = f"on_{type}"
+        if type == EventType.TITLE_UPDATE:
+            self._listener.dispatch(event_name, TitleUpdateEvent(**data[type]))
+        elif type == EventType.PLAYLIST_UPDATE:
+            event_model = PlayListUpdateEvent(**data[type])
+            self._listener.dispatch(event_name, event_model)
+            await self.__dispatch_new_series(event_model)
+        elif type in [
+            EventType.ENCODE_START,
+            EventType.ENCODE_END,
+            EventType.ENCODE_PROGRESS,
+            EventType.ENCODE_FINISH,
+        ]:
+            self._listener.dispatch(event_name, EncodeEvent(**data[type]))
+        elif type == EventType.TORRENT_UPDATE:
+            self._listener.dispatch(event_name, TorrentUpdateEvent(**data[type]))
+        else:
+            self._listener.dispatch(event_name, data)
+            log.debug(f"Not documented event type {type} dispatched with data: {data}")
+
+    async def __dispatch_new_series(self, event_model: PlayListUpdateEvent):
         """
-        Диспатчит ивент ``on_connect``
+        Диспатчит ивент ``on_title_serie``
+
+        .. warning::
+           Не пытайтесь самостоятельно использовать этот метод!
+
+        :param event_model:
+        :return:
+        """
+        if not event_model.updated_episode:
+            return
+        hls = event_model.updated_episode.hls
+        if not hls.fhd or not hls.hd or not hls.sd or event_model.reupload:
+            return
+        title = await self._http.v2.get_title(id=event_model.id)
+        event_model = TitleSerieEvent(title=title, episode=event_model.updated_episode)
+        self._listener.dispatch("on_title_serie", event_model)
+
+    async def __dispatch_other_events(self, data: dict):
+        """
+        Диспатчит ивент ``on_connect``.
 
         .. warning::
            Не пытайтесь самостоятельно использовать этот метод!
@@ -141,27 +155,4 @@ class WebSocketClient:
             log.debug(f"Successfully connected to API. API version {api_version}")
             self._listener.dispatch("on_connect")
         else:
-            log.debug(data)
-
-    async def __subscribe_on_titles(self):
-        """
-        Подписывается на все тайтлы.
-
-        .. warning::
-           Не пытайтесь самостоятельно использовать этот метод!
-        """
-        for subscribe in self._subscribes:
-            await self._subscribe(subscribe)
-        self._subscribes = None
-
-    async def _subscribe(self, data: dict):
-        """
-        Подписывается на тайтл, отправкой пакета
-
-        .. warning::
-           Не пытайтесь самостоятельно использовать этот метод!
-
-        :param data: Словарь с данными о подписке.
-        """
-        await self._client.send_json(data)
-        log.debug(f"Send json to websocket with data: {data}")
+            log.debug(f"Not documented event data: {data}")
