@@ -3,7 +3,10 @@ from .v2 import V2Request
 from .public import PublicRequest
 
 
-class HTTPCLient:
+__all__ = ["HTTPClient"]
+
+
+class HTTPClient:
     def __init__(self, proxy: str = None) -> None:
         """
 
